@@ -16,36 +16,55 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  * USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
 
+#include <algorithm>
+
+#include <core/LogWriter.h>
+#include <core/string.h>
+
 #include <rfb/Exception.h>
 #include <rfb/clipboardTypes.h>
 #include <rfb/fenceTypes.h>
+#include <rfb/screenTypes.h>
 #include <rfb/CMsgReader.h>
 #include <rfb/CMsgWriter.h>
 #include <rfb/CSecurity.h>
+#include <rfb/Cursor.h>
 #include <rfb/Decoder.h>
+#include <rfb/KeysymStr.h>
+#include <rfb/PixelBuffer.h>
 #include <rfb/Security.h>
 #include <rfb/SecurityClient.h>
 #include <rfb/CConnection.h>
-#include <rfb/util.h>
 
-#include <rfb/LogWriter.h>
+#define XK_MISCELLANY
+#define XK_XKB_KEYS
+#include <rfb/keysymdef.h>
 
 #include <rdr/InStream.h>
 #include <rdr/OutStream.h>
 
 using namespace rfb;
 
-static LogWriter vlog("CConnection");
+static core::LogWriter vlog("CConnection");
+
+core::BoolParameter
+  CConnection::noJpeg("NoJPEG", "Disable lossy JPEG compression.",
+                      false);
 
 CConnection::CConnection()
-  : csecurity(0),
+  : csecurity(nullptr),
     supportsLocalCursor(false), supportsCursorPosition(false),
     supportsDesktopResize(false), supportsLEDState(false),
-    is(0), os(0), reader_(0), writer_(0),
+    is(nullptr), os(nullptr), reader_(nullptr), writer_(nullptr),
     shared(false),
     state_(RFBSTATE_UNINITIALISED),
     pendingPFChange(false), preferredEncoding(encodingTight),
@@ -53,8 +72,8 @@ CConnection::CConnection()
     formatChange(false), encodingChange(false),
     firstUpdate(true), pendingUpdate(false), continuousUpdates(false),
     forceNonincremental(true),
-    framebuffer(NULL), decoder(this),
-    serverClipboard(NULL), hasLocalClipboard(false)
+    framebuffer(nullptr), decoder(this),
+    hasRemoteClipboard(false), hasLocalClipboard(false)
 {
 }
 
@@ -63,6 +82,13 @@ CConnection::~CConnection()
   close();
 }
 
+void CConnection::setServerName(const char* name_)
+{
+  if (name_ == nullptr)
+    name_ = "";
+  serverName = name_;
+}
+
 void CConnection::setStreams(rdr::InStream* is_, rdr::OutStream* os_)
 {
   is = is_;
@@ -78,19 +104,18 @@ void CConnection::setFramebuffer(ModifiablePixelBuffer* fb)
     assert(fb->height() == server.height());
   }
 
-  if ((framebuffer != NULL) && (fb != NULL)) {
-    Rect rect;
+  if ((framebuffer != nullptr) && (fb != nullptr)) {
+    core::Rect rect;
 
-    const rdr::U8* data;
+    const uint8_t* data;
     int stride;
 
-    const rdr::U8 black[4] = { 0, 0, 0, 0 };
+    const uint8_t black[4] = { 0, 0, 0, 0 };
 
     // Copy still valid area
 
-    rect.setXYWH(0, 0,
-                 __rfbmin(fb->width(), framebuffer->width()),
-                 __rfbmin(fb->height(), framebuffer->height()));
+    rect = fb->getRect();
+    rect = rect.intersect(framebuffer->getRect());
     data = framebuffer->getBuffer(framebuffer->getRect(), &stride);
     fb->imageRect(rect, data, stride);
 
@@ -132,11 +157,11 @@ bool CConnection::processMsg()
   case RFBSTATE_INITIALISATION:   return processInitMsg();           break;
   case RFBSTATE_NORMAL:           return reader_->readMsg();         break;
   case RFBSTATE_CLOSING:
-    throw Exception("CConnection::processMsg: called while closing");
+    throw std::logic_error("CConnection::processMsg: Called while closing");
   case RFBSTATE_UNINITIALISED:
-    throw Exception("CConnection::processMsg: not initialised yet?");
+    throw std::logic_error("CConnection::processMsg: Not initialised yet?");
   default:
-    throw Exception("CConnection::processMsg: invalid state");
+    throw std::logic_error("CConnection::processMsg: Invalid state");
   }
 }
 
@@ -146,18 +171,18 @@ bool CConnection::processVersionMsg()
   int majorVersion;
   int minorVersion;
 
-  vlog.debug("reading protocol version");
+  vlog.debug("Reading protocol version");
 
   if (!is->hasData(12))
     return false;
 
-  is->readBytes(verStr, 12);
+  is->readBytes((uint8_t*)verStr, 12);
   verStr[12] = '\0';
 
   if (sscanf(verStr, "RFB %03d.%03d\n",
              &majorVersion, &minorVersion) != 2) {
     state_ = RFBSTATE_INVALID;
-    throw Exception("reading version failed: not an RFB server?");
+    throw protocol_error("Reading version failed, not an RFB server?");
   }
 
   server.setVersion(majorVersion, minorVersion);
@@ -170,8 +195,9 @@ bool CConnection::processVersionMsg()
     vlog.error("Server gave unsupported RFB protocol version %d.%d",
                server.majorVersion, server.minorVersion);
     state_ = RFBSTATE_INVALID;
-    throw Exception("Server gave unsupported RFB protocol version %d.%d",
-                    server.majorVersion, server.minorVersion);
+    throw protocol_error(
+      core::format("Server gave unsupported RFB protocol version %d.%d",
+                   server.majorVersion, server.minorVersion));
   } else if (server.beforeVersion(3,7)) {
     server.setVersion(3,3);
   } else if (server.afterVersion(3,8)) {
@@ -180,7 +206,7 @@ bool CConnection::processVersionMsg()
 
   sprintf(verStr, "RFB %03d.%03d\n",
           server.majorVersion, server.minorVersion);
-  os->writeBytes(verStr, 12);
+  os->writeBytes((const uint8_t*)verStr, 12);
   os->flush();
 
   state_ = RFBSTATE_SECURITY_TYPES;
@@ -194,11 +220,11 @@ bool CConnection::processVersionMsg()
 
 bool CConnection::processSecurityTypesMsg()
 {
-  vlog.debug("processing security types message");
+  vlog.debug("Processing security types message");
 
   int secType = secTypeInvalid;
 
-  std::list<rdr::U8> secTypes;
+  std::list<uint8_t> secTypes;
   secTypes = security.GetEnabledSecTypes();
 
   if (server.isVersion(3,3)) {
@@ -213,18 +239,12 @@ bool CConnection::processSecurityTypesMsg()
       state_ = RFBSTATE_SECURITY_REASON;
       return true;
     } else if (secType == secTypeNone || secType == secTypeVncAuth) {
-      std::list<rdr::U8>::iterator i;
-      for (i = secTypes.begin(); i != secTypes.end(); i++)
-        if (*i == secType) {
-          secType = *i;
-          break;
-        }
-
-      if (i == secTypes.end())
+      if (std::find(secTypes.begin(), secTypes.end(),
+                    secType) == secTypes.end())
         secType = secTypeInvalid;
     } else {
       vlog.error("Unknown 3.3 security type %d", secType);
-      throw Exception("Unknown 3.3 security type");
+      throw protocol_error("Unknown 3.3 security type");
     }
 
   } else {
@@ -247,10 +267,8 @@ bool CConnection::processSecurityTypesMsg()
       return true;
     }
 
-    std::list<rdr::U8>::iterator j;
-
     for (int i = 0; i < nServerSecTypes; i++) {
-      rdr::U8 serverSecType = is->readU8();
+      uint8_t serverSecType = is->readU8();
       vlog.debug("Server offers security type %s(%d)",
                  secTypeName(serverSecType), serverSecType);
 
@@ -259,12 +277,10 @@ bool CConnection::processSecurityTypesMsg()
        * It means server's order specifies priority.
        */
       if (secType == secTypeInvalid) {
-        for (j = secTypes.begin(); j != secTypes.end(); j++)
-          if (*j == serverSecType) {
-            secType = *j;
-            break;
-          }
-       }
+        if (std::find(secTypes.begin(), secTypes.end(),
+                      serverSecType) != secTypes.end())
+          secType = serverSecType;
+      }
     }
 
     // Inform the server of our decision
@@ -278,7 +294,7 @@ bool CConnection::processSecurityTypesMsg()
   if (secType == secTypeInvalid) {
     state_ = RFBSTATE_INVALID;
     vlog.error("No matching security types");
-    throw Exception("No matching security types");
+    throw protocol_error("No matching security types");
   }
 
   state_ = RFBSTATE_SECURITY;
@@ -289,7 +305,7 @@ bool CConnection::processSecurityTypesMsg()
 
 bool CConnection::processSecurityMsg()
 {
-  vlog.debug("processing security message");
+  vlog.debug("Processing security message");
   if (!csecurity->processMsg())
     return false;
 
@@ -300,7 +316,7 @@ bool CConnection::processSecurityMsg()
 
 bool CConnection::processSecurityResultMsg()
 {
-  vlog.debug("processing security result message");
+  vlog.debug("Processing security result message");
   int result;
 
   if (server.beforeVersion(3,8) && csecurity->getType() == secTypeNone) {
@@ -316,18 +332,18 @@ bool CConnection::processSecurityResultMsg()
     securityCompleted();
     return true;
   case secResultFailed:
-    vlog.debug("auth failed");
+    vlog.debug("Auth failed");
     break;
   case secResultTooMany:
-    vlog.debug("auth failed - too many tries");
+    vlog.debug("Auth failed: Too many tries");
     break;
   default:
-    throw Exception("Unknown security result from server");
+    throw protocol_error("Unknown security result from server");
   }
 
   if (server.beforeVersion(3,8)) {
     state_ = RFBSTATE_INVALID;
-    throw AuthFailureException();
+    throw auth_error("Authentication failed");
   }
 
   state_ = RFBSTATE_SECURITY_REASON;
@@ -336,29 +352,29 @@ bool CConnection::processSecurityResultMsg()
 
 bool CConnection::processSecurityReasonMsg()
 {
-  vlog.debug("processing security reason message");
+  vlog.debug("Processing security reason message");
 
   if (!is->hasData(4))
     return false;
 
   is->setRestorePoint();
 
-  rdr::U32 len = is->readU32();
+  uint32_t len = is->readU32();
   if (!is->hasDataOrRestore(len))
     return false;
   is->clearRestorePoint();
 
-  CharArray reason(len + 1);
-  is->readBytes(reason.buf, len);
-  reason.buf[len] = '\0';
+  std::vector<char> reason(len + 1);
+  is->readBytes((uint8_t*)reason.data(), len);
+  reason[len] = '\0';
 
   state_ = RFBSTATE_INVALID;
-  throw AuthFailureException(reason.buf);
+  throw auth_error(reason.data());
 }
 
 bool CConnection::processInitMsg()
 {
-  vlog.debug("reading server initialisation");
+  vlog.debug("Reading server initialisation");
   return reader_->readServerInit();
 }
 
@@ -368,7 +384,6 @@ void CConnection::securityCompleted()
   reader_ = new CMsgReader(this, is);
   writer_ = new CMsgWriter(&server, os);
   vlog.debug("Authentication success!");
-  authSuccess();
   writer_->writeClientInit(shared);
 }
 
@@ -376,22 +391,30 @@ void CConnection::close()
 {
   state_ = RFBSTATE_CLOSING;
 
-  setFramebuffer(NULL);
+  /*
+   * We're already shutting down, so just log any pending decoder
+   * problems
+   */
+  try {
+    decoder.flush();
+  } catch (std::exception& e) {
+    vlog.error("%s", e.what());
+  }
+
+  setFramebuffer(nullptr);
   delete csecurity;
-  csecurity = NULL;
+  csecurity = nullptr;
   delete reader_;
-  reader_ = NULL;
+  reader_ = nullptr;
   delete writer_;
-  writer_ = NULL;
-  strFree(serverClipboard);
-  serverClipboard = NULL;
+  writer_ = nullptr;
 }
 
 void CConnection::setDesktopSize(int w, int h)
 {
   decoder.flush();
 
-  CMsgHandler::setDesktopSize(w,h);
+  server.setDimensions(w, h);
 
   if (continuousUpdates)
     writer()->writeEnableContinuousUpdates(true, 0, 0,
@@ -399,7 +422,7 @@ void CConnection::setDesktopSize(int w, int h)
                                            server.height());
 
   resizeFramebuffer();
-  assert(framebuffer != NULL);
+  assert(framebuffer != nullptr);
   assert(framebuffer->width() == server.width());
   assert(framebuffer->height() == server.height());
 }
@@ -411,7 +434,15 @@ void CConnection::setExtendedDesktopSize(unsigned reason,
 {
   decoder.flush();
 
-  CMsgHandler::setExtendedDesktopSize(reason, result, w, h, layout);
+  server.supportsSetDesktopSize = true;
+
+  if ((reason != reasonClient) || (result == resultSuccess))
+    server.setDimensions(w, h, layout);
+
+  if ((reason == reasonClient) && (result != resultSuccess)) {
+    vlog.error("SetDesktopSize failed: %d", result);
+    return;
+  }
 
   if (continuousUpdates)
     writer()->writeEnableContinuousUpdates(true, 0, 0,
@@ -419,14 +450,46 @@ void CConnection::setExtendedDesktopSize(unsigned reason,
                                            server.height());
 
   resizeFramebuffer();
-  assert(framebuffer != NULL);
+  assert(framebuffer != nullptr);
   assert(framebuffer->width() == server.width());
   assert(framebuffer->height() == server.height());
 }
 
+void CConnection::setCursor(int width, int height,
+                            const core::Point& hotspot,
+                            const uint8_t* data)
+{
+  Cursor cursor(width, height, hotspot, data);
+  server.setCursor(cursor);
+}
+
+void CConnection::setCursorPos(const core::Point& /*pos*/)
+{
+}
+
+void CConnection::setName(const char* name)
+{
+  server.setName(name);
+}
+
+void CConnection::fence(uint32_t flags, unsigned len, const uint8_t data[])
+{
+  server.supportsFence = true;
+
+  if (flags & fenceFlagRequest) {
+    // FIXME: We handle everything synchronously, and we assume anything
+    //        using us also does so, which means we automatically handle
+    //        these flags
+    flags = flags & (fenceFlagBlockBefore | fenceFlagBlockAfter);
+
+    writer()->writeFence(flags, len, data);
+    return;
+  }
+}
+
 void CConnection::endOfContinuousUpdates()
 {
-  CMsgHandler::endOfContinuousUpdates();
+  server.supportsContinuousUpdates = true;
 
   // We've gotten the marker for a format change, so make the pending
   // one active
@@ -440,17 +503,29 @@ void CConnection::endOfContinuousUpdates()
   }
 }
 
+void CConnection::supportsQEMUKeyEvent()
+{
+  server.supportsQEMUKeyEvent = true;
+}
+
+void CConnection::supportsExtendedMouseButtons()
+{
+  server.supportsExtendedMouseButtons = true;
+}
+
 void CConnection::serverInit(int width, int height,
                              const PixelFormat& pf,
                              const char* name)
 {
-  CMsgHandler::serverInit(width, height, pf, name);
+  server.setDimensions(width, height);
+  server.setPF(pf);
+  server.setName(name);
 
   state_ = RFBSTATE_NORMAL;
-  vlog.debug("initialisation done");
+  vlog.debug("Initialisation done");
 
   initDone();
-  assert(framebuffer != NULL);
+  assert(framebuffer != nullptr);
   assert(framebuffer->width() == server.width());
   assert(framebuffer->height() == server.height());
 
@@ -467,7 +542,7 @@ void CConnection::serverInit(int width, int height,
   }
 }
 
-bool CConnection::readAndDecodeRect(const Rect& r, int encoding,
+bool CConnection::readAndDecodeRect(const core::Rect& r, int encoding,
                                     ModifiablePixelBuffer* pb)
 {
   if (!decoder.decodeRect(r, encoding, pb))
@@ -478,9 +553,7 @@ bool CConnection::readAndDecodeRect(const Rect& r, int encoding,
 
 void CConnection::framebufferUpdateStart()
 {
-  CMsgHandler::framebufferUpdateStart();
-
-  assert(framebuffer != NULL);
+  assert(framebuffer != nullptr);
 
   // Note: This might not be true if continuous updates are supported
   pendingUpdate = false;
@@ -492,8 +565,6 @@ void CConnection::framebufferUpdateEnd()
 {
   decoder.flush();
 
-  CMsgHandler::framebufferUpdateEnd();
-
   // A format change has been scheduled and we are now past the update
   // with the old format. Time to active the new one.
   if (pendingPFChange && !continuousUpdates) {
@@ -514,29 +585,75 @@ void CConnection::framebufferUpdateEnd()
   }
 }
 
-bool CConnection::dataRect(const Rect& r, int encoding)
+bool CConnection::dataRect(const core::Rect& r, int encoding)
 {
   return decoder.decodeRect(r, encoding, framebuffer);
 }
 
+void CConnection::setColourMapEntries(int /*firstColour*/,
+                                      int /*nColours*/,
+                                      uint16_t* /*rgbs*/)
+{
+  vlog.error("Invalid SetColourMapEntries from server!");
+}
+
 void CConnection::serverCutText(const char* str)
 {
   hasLocalClipboard = false;
 
-  strFree(serverClipboard);
-  serverClipboard = NULL;
-
-  serverClipboard = latin1ToUTF8(str);
+  serverClipboard = str;
+  hasRemoteClipboard = true;
 
   handleClipboardAnnounce(true);
 }
 
-void CConnection::handleClipboardCaps(rdr::U32 flags,
-                                      const rdr::U32* lengths)
+void CConnection::setLEDState(unsigned int state)
 {
-  rdr::U32 sizes[] = { 0 };
+  server.setLEDState(state);
+}
 
-  CMsgHandler::handleClipboardCaps(flags, lengths);
+void CConnection::handleClipboardCaps(uint32_t flags,
+                                      const uint32_t* lengths)
+{
+  int i;
+  uint32_t sizes[] = { 0 };
+
+  vlog.debug("Got server clipboard capabilities:");
+  for (i = 0;i < 16;i++) {
+    if (flags & (1 << i)) {
+      const char *type;
+
+      switch (1 << i) {
+        case clipboardUTF8:
+          type = "Plain text";
+          break;
+        case clipboardRTF:
+          type = "Rich text";
+          break;
+        case clipboardHTML:
+          type = "HTML";
+          break;
+        case clipboardDIB:
+          type = "Images";
+          break;
+        case clipboardFiles:
+          type = "Files";
+          break;
+        default:
+          vlog.debug("    Unknown format 0x%x", 1 << i);
+          continue;
+      }
+
+      if (lengths[i] == 0)
+        vlog.debug("    %s (only notify)", type);
+      else {
+        vlog.debug("    %s (automatically send up to %s)",
+                   type, core::iecPrefix(lengths[i], "B").c_str());
+      }
+    }
+  }
+
+  server.setClipboardCaps(flags, lengths);
 
   writer()->writeClipboardCaps(rfb::clipboardUTF8 |
                                rfb::clipboardRequest |
@@ -546,7 +663,7 @@ void CConnection::handleClipboardCaps(rdr::U32 flags,
                                sizes);
 }
 
-void CConnection::handleClipboardRequest(rdr::U32 flags)
+void CConnection::handleClipboardRequest(uint32_t flags)
 {
   if (!(flags & rfb::clipboardUTF8)) {
     vlog.debug("Ignoring clipboard request for unsupported formats 0x%x", flags);
@@ -559,16 +676,15 @@ void CConnection::handleClipboardRequest(rdr::U32 flags)
   handleClipboardRequest();
 }
 
-void CConnection::handleClipboardPeek(rdr::U32 flags)
+void CConnection::handleClipboardPeek()
 {
   if (server.clipboardFlags() & rfb::clipboardNotify)
     writer()->writeClipboardNotify(hasLocalClipboard ? rfb::clipboardUTF8 : 0);
 }
 
-void CConnection::handleClipboardNotify(rdr::U32 flags)
+void CConnection::handleClipboardNotify(uint32_t flags)
 {
-  strFree(serverClipboard);
-  serverClipboard = NULL;
+  hasRemoteClipboard = false;
 
   if (flags & rfb::clipboardUTF8) {
     hasLocalClipboard = false;
@@ -578,26 +694,25 @@ void CConnection::handleClipboardNotify(rdr::U32 flags)
   }
 }
 
-void CConnection::handleClipboardProvide(rdr::U32 flags,
+void CConnection::handleClipboardProvide(uint32_t flags,
                                          const size_t* lengths,
-                                         const rdr::U8* const* data)
+                                         const uint8_t* const* data)
 {
   if (!(flags & rfb::clipboardUTF8)) {
     vlog.debug("Ignoring clipboard provide with unsupported formats 0x%x", flags);
     return;
   }
 
-  strFree(serverClipboard);
-  serverClipboard = NULL;
-
-  serverClipboard = convertLF((const char*)data[0], lengths[0]);
+  // FIXME: This conversion magic should be in CMsgReader
+  if (!core::isValidUTF8((const char*)data[0], lengths[0])) {
+    vlog.error("Invalid UTF-8 sequence in clipboard - ignoring");
+    return;
+  }
+  serverClipboard = core::convertLF((const char*)data[0], lengths[0]);
+  hasRemoteClipboard = true;
 
   // FIXME: Should probably verify that this data was actually requested
-  handleClipboardData(serverClipboard);
-}
-
-void CConnection::authSuccess()
-{
+  handleClipboardData(serverClipboard.c_str());
 }
 
 void CConnection::initDone()
@@ -613,18 +728,18 @@ void CConnection::handleClipboardRequest()
 {
 }
 
-void CConnection::handleClipboardAnnounce(bool available)
+void CConnection::handleClipboardAnnounce(bool /*available*/)
 {
 }
 
-void CConnection::handleClipboardData(const char* data)
+void CConnection::handleClipboardData(const char* /*data*/)
 {
 }
 
 void CConnection::requestClipboard()
 {
-  if (serverClipboard != NULL) {
-    handleClipboardData(serverClipboard);
+  if (hasRemoteClipboard) {
+    handleClipboardData(serverClipboard.c_str());
     return;
   }
 
@@ -659,9 +774,10 @@ void CConnection::announceClipboard(bool available)
 void CConnection::sendClipboardData(const char* data)
 {
   if (server.clipboardFlags() & rfb::clipboardProvide) {
-    CharArray filtered(convertCRLF(data));
-    size_t sizes[1] = { strlen(filtered.buf) + 1 };
-    const rdr::U8* data[1] = { (const rdr::U8*)filtered.buf };
+    // FIXME: This conversion magic should be in CMsgWriter
+    std::string filtered(core::convertCRLF(data));
+    size_t sizes[1] = { filtered.size() + 1 };
+    const uint8_t* datas[1] = { (const uint8_t*)filtered.c_str() };
 
     if (unsolicitedClipboardAttempt) {
       unsolicitedClipboardAttempt = false;
@@ -673,14 +789,95 @@ void CConnection::sendClipboardData(const char* data)
       }
     }
 
-    writer()->writeClipboardProvide(rfb::clipboardUTF8, sizes, data);
+    writer()->writeClipboardProvide(rfb::clipboardUTF8, sizes, datas);
   } else {
-    CharArray latin1(utf8ToLatin1(data));
-
-    writer()->writeClientCutText(latin1.buf);
+    writer()->writeClientCutText(data);
   }
 }
 
+void CConnection::sendKeyPress(int systemKeyCode,
+                               uint32_t keyCode, uint32_t keySym)
+{
+  // For the first few years, there wasn't a good consensus on what the
+  // Windows keys should be mapped to for X11. So we need to help out a
+  // bit and map all variants to the same key...
+  switch (keySym) {
+  case XK_Hyper_L:
+    keySym = XK_Super_L;
+    break;
+  case XK_Hyper_R:
+    keySym = XK_Super_R;
+    break;
+  // There has been several variants for Shift-Tab over the years.
+  // RFB states that we should always send a normal tab.
+  case XK_ISO_Left_Tab:
+    keySym = XK_Tab;
+    break;
+  }
+
+#ifdef __APPLE__
+  // Alt on OS X behaves more like AltGr on other systems, and to get
+  // sane behaviour we should translate things in that manner for the
+  // remote VNC server. However that means we lose the ability to use
+  // Alt as a shortcut modifier. Do what RealVNC does and hijack the
+  // left command key as an Alt replacement.
+  switch (keySym) {
+  case XK_Super_L:
+    keySym = XK_Alt_L;
+    break;
+  case XK_Super_R:
+    keySym = XK_Super_L;
+    break;
+  case XK_Alt_L:
+    keySym = XK_Mode_switch;
+    break;
+  case XK_Alt_R:
+    keySym = XK_ISO_Level3_Shift;
+    break;
+  }
+#endif
+
+  // Because of the way keyboards work, we cannot expect to have the same
+  // symbol on release as when pressed. This breaks the VNC protocol however,
+  // so we need to keep track of what keysym a key _code_ generated on press
+  // and send the same on release.
+  downKeys[systemKeyCode].keyCode = keyCode;
+  downKeys[systemKeyCode].keySym = keySym;
+
+  vlog.debug("Key pressed: %d => 0x%02x / XK_%s (0x%04x)",
+             systemKeyCode, keyCode, KeySymName(keySym), keySym);
+
+  writer()->writeKeyEvent(keySym, keyCode, true);
+}
+
+void CConnection::sendKeyRelease(int systemKeyCode)
+{
+  DownMap::iterator iter;
+
+  iter = downKeys.find(systemKeyCode);
+  if (iter == downKeys.end()) {
+    // These occur somewhat frequently so let's not spam them unless
+    // logging is turned up.
+    vlog.debug("Unexpected release of key code %d", systemKeyCode);
+    return;
+  }
+
+  vlog.debug("Key released: %d => 0x%02x / XK_%s (0x%04x)",
+             systemKeyCode, iter->second.keyCode,
+             KeySymName(iter->second.keySym), iter->second.keySym);
+
+  writer()->writeKeyEvent(iter->second.keySym,
+                          iter->second.keyCode, false);
+
+  downKeys.erase(iter);
+}
+
+void CConnection::releaseAllKeys()
+{
+  while (!downKeys.empty())
+    sendKeyRelease(downKeys.begin()->first);
+}
+
 void CConnection::refreshFramebuffer()
 {
   forceNonincremental = true;
@@ -714,6 +911,11 @@ void CConnection::setCompressLevel(int level)
   encodingChange = true;
 }
 
+int CConnection::getCompressLevel()
+{
+  return compressLevel;
+}
+
 void CConnection::setQualityLevel(int level)
 {
   if (qualityLevel == level)
@@ -723,26 +925,23 @@ void CConnection::setQualityLevel(int level)
   encodingChange = true;
 }
 
+int CConnection::getQualityLevel()
+{
+  return qualityLevel;
+}
+
 void CConnection::setPF(const PixelFormat& pf)
 {
-  if (server.pf().equal(pf) && !formatChange)
+  if (server.pf() == pf && !formatChange)
     return;
 
   nextPF = pf;
   formatChange = true;
 }
 
-void CConnection::fence(rdr::U32 flags, unsigned len, const char data[])
+bool CConnection::isSecure() const
 {
-  CMsgHandler::fence(flags, len, data);
-
-  if (!(flags & fenceFlagRequest))
-    return;
-
-  // We cannot guarantee any synchronisation at this level
-  flags = 0;
-
-  writer()->writeFence(flags, len, data);
+  return csecurity ? csecurity->isSecure() : false;
 }
 
 // requestNewUpdate() requests an update from the server, having set the
@@ -783,9 +982,9 @@ void CConnection::requestNewUpdate()
 
   if (forceNonincremental || !continuousUpdates) {
     pendingUpdate = true;
-    writer()->writeFramebufferUpdateRequest(Rect(0, 0,
-                                                 server.width(),
-                                                 server.height()),
+    writer()->writeFramebufferUpdateRequest({0, 0,
+                                             server.width(),
+                                             server.height()},
                                             !forceNonincremental);
   }
 
@@ -797,7 +996,7 @@ void CConnection::requestNewUpdate()
 
 void CConnection::updateEncodings()
 {
-  std::list<rdr::U32> encodings;
+  std::list<uint32_t> encodings;
 
   if (supportsLocalCursor) {
     encodings.push_back(pseudoEncodingCursorWithAlpha);
@@ -823,22 +1022,30 @@ void CConnection::updateEncodings()
   encodings.push_back(pseudoEncodingContinuousUpdates);
   encodings.push_back(pseudoEncodingFence);
   encodings.push_back(pseudoEncodingQEMUKeyEvent);
+  encodings.push_back(pseudoEncodingExtendedMouseButtons);
 
   if (Decoder::supported(preferredEncoding)) {
-    encodings.push_back(preferredEncoding);
+    if (!noJpeg || preferredEncoding != encodingJPEG)
+      encodings.push_back(preferredEncoding);
   }
 
   encodings.push_back(encodingCopyRect);
 
   for (int i = encodingMax; i >= 0; i--) {
-    if ((i != preferredEncoding) && Decoder::supported(i))
+    if ((i != preferredEncoding) && Decoder::supported(i)) {
+      if (noJpeg && i == encodingJPEG)
+        continue;
       encodings.push_back(i);
+    }
   }
 
   if (compressLevel >= 0 && compressLevel <= 9)
       encodings.push_back(pseudoEncodingCompressLevel0 + compressLevel);
-  if (qualityLevel >= 0 && qualityLevel <= 9)
+  // Tight JPEG is enabled by setting a quality level
+  if (!noJpeg) {
+    if (qualityLevel >= 0 && qualityLevel <= 9)
       encodings.push_back(pseudoEncodingQualityLevel0 + qualityLevel);
+  }
 
   writer()->writeSetEncodings(encodings);
 }
