@@ -24,8 +24,8 @@
 #ifndef __RFB_WIN32_INPUT_H__
 #define __RFB_WIN32_INPUT_H__
 
-#include <rfb/Rect.h>
-#include <rfb/Configuration.h>
+#include <core/Configuration.h>
+#include <core/Rect.h>
 
 #include <map>
 #include <vector>
@@ -44,9 +44,9 @@ namespace rfb {
       // - Create a pointer event at a the given coordinates, with the
       //   specified button state.  The event must be specified using
       //   Screen coordinates.
-      void pointerEvent(const Point& pos, uint16_t buttonmask);
+      void pointerEvent(const core::Point& pos, uint16_t buttonmask);
     protected:
-      Point last_position;
+      core::Point last_position;
       uint16_t last_buttonmask;
     };
 
@@ -56,8 +56,8 @@ namespace rfb {
     public:
       SKeyboard();
       void keyEvent(uint32_t keysym, uint32_t keycode, bool down);
-      static BoolParameter deadKeyAware;
-      static BoolParameter rawKeyboard;
+      static core::BoolParameter deadKeyAware;
+      static core::BoolParameter rawKeyboard;
     private:
       std::map<uint32_t,uint8_t> vkMap;
       std::map<uint32_t,bool> extendedMap;
