@@ -21,9 +21,8 @@
 #include <config.h>
 #endif
 
-#include <rdr/Exception.h>
+#include <core/LogWriter.h>
 
-#include <rfb/LogWriter.h>
 #include <rfb/Security.h>
 
 #include <rfb_win32/resource.h>
@@ -31,7 +30,7 @@
 
 #include <list>
 
-using namespace rdr;
+using namespace core;
 using namespace rfb;
 using namespace rfb::win32;
 using namespace std;
@@ -124,9 +123,6 @@ SecurityPage::onOk() {
   bool vnc_loaded = false;
   list<uint32_t> secTypes;
 
-  /* Keep same priorities as in common/rfb/SecurityClient::secTypes */
-  secTypes.push_back(secTypeVeNCrypt);
-
 #ifdef HAVE_GNUTLS
   /* X509Plain */
   if (authMethodEnabled(IDC_ENC_X509, IDC_AUTH_PLAIN)) {
